/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: chatsndwin.cpp,v 1.12 2001/01/17 02:01:54 bills Exp $
// $Log: chatsndwin.cpp,v $
// Revision 1.12  2001/01/17 02:01:54  bills
// port kicq to new chat/file session interface, add icqlib subdir
//
// Revision 1.11  2000/12/19 06:17:06  bills
// change icqlink member in regwin and mainwindow to an ICQLINK *, use
// icq_ICQLINKNew and icq_ICQLINKDelete instead of icq_Init and icq_Done
//
// Revision 1.10  2000/10/14 21:00:49  davidvh
// Minor fixes.
//
// Revision 1.9  2000/10/10 14:46:16  davidvh
// Replaced the CTRL-<ENTER> shortcut for 'Send' with ALT-S.
// Made the shortcut for cancel ALT-C.
// Made variable names easier to read and fixed my styling to match Denis'.
//
// Revision 1.8  2000/10/02 07:37:42  denis
// First two patches from David van Hoose <david-vh@home.com> with some
// modifications were applied.
//
// Revision 1.7  2000/09/30 20:05:40  napalmbox
// big update
//
// Revision 1.6  2000/09/18 11:23:29  napalmbox
// fix
//
// Revision 1.5  2000/09/13 19:11:12  napalmbox
// replase kicq ver 1.0.0 files with kicq ver 2.0.0 files
//
// Revision 1.3.2.2  2000/07/22 19:44:03  napalmbox
// kicq compiles now with -DQT_NO_ASCII_CAST
//
// Revision 1.3.2.1  2000/07/02 05:45:54  napalmbox
// port kicq -> kde2 :))
//
// Revision 1.3  2000/06/17 17:15:26  denis
// Cleanups.
//
// Revision 1.2  2000/06/15 01:55:56  bills
// fixed to use renamed chat functions
//
// Revision 1.1  2000/05/24 16:56:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.11  2000/01/10 16:08:04  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.10  1999/12/02 13:58:02  denis
// Cleanups.
// IcqWindow random segfaults fixed.
//
// Revision 1.9  1999/10/06 19:10:51  denis
// Cleanups.
//
// Revision 1.8  1999/10/04 16:48:26  bills
// ICQ_NOTIFY_CHATINIT -> ICQ_NOTIFY_CHAT
//
// Revision 1.7  1999/10/01 21:53:03  guruz
// *** empty log message ***
//
// Revision 1.6  1999/07/16 15:47:41  denis
// Cleaned up.
//
// Revision 1.5  1999/07/16 12:59:36  denis
// Internalization support added.
// Smart loading of spinning face animation implemented.
//
// Revision 1.4  1999/07/12 15:23:17  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.3  1999/04/17 20:22:11  bills
// modified for new icqwindow base class
//
// Revision 1.2  1999/04/14 15:35:05  denis
// Commented out unused variable in sendChatReq()
//
// Revision 1.1  1999/04/04 21:57:11  bills
// added for chat support
//
// Revision 1.8  1999/03/31 01:27:08  bills
// added tcpflag check to sendURL
//
// Revision 1.7  1999/03/28 02:02:37  bills
// fixed number of args to icq_SendURL
//
// Revision 1.6  1998/12/08 16:33:51  denis
// Cleaned up a little before releasing
//

#include <icq.h>
#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include "main.h"

#include "chatwindow.h"
#include "chatsndwin.h"
#include "mainwindow.h"
#include "userlist.h"
#include "icqmessage.h"

#include "chatsndwin.moc"

ChatSndWindow::ChatSndWindow(const QString &n, unsigned long u, const QString &em) : IcqWindow()
{
  CrashGuard();
  setIcon(UserIcon("chat"));
  QString text;
  QPixmap infopix;
  setCaption(i18n("Send Chat Request"));

  toBox = new QGroupBox(i18n("To"), this);
  sendBtn = new QPushButton(i18n("&Send"), this);
// Changed accelerated keystroke to ALT-S
//  sendBtn->setAccel(Key_Return+CTRL);
  connect(sendBtn, SIGNAL(clicked()), SLOT(sendChatReq()));
  cancelBtn = new QPushButton(i18n("&Cancel"), this);
  connect(cancelBtn, SIGNAL(clicked()), SLOT(cancelChatReq()));
  uinLine = new QLineEdit(this);
  uinLine->setFocusPolicy(QWidget::NoFocus);
  uinLabel = new QLabel(uinLine,i18n("ICQ #:"), this);
  handleLine = new QLineEdit(this);
  handleLine->setFocusPolicy(QWidget::NoFocus);
  handleLabel = new QLabel(handleLine,i18n("Handle:"), this);
  emailLine = new QLineEdit(this);
  emailLine->setFocusPolicy(QWidget::NoFocus);
  emailLabel = new QLabel(emailLine,i18n("E-Mail:"), this);
  infoBtn = new QPushButton(this);
  infopix = UserIcon("info2");
  infoBtn->setPixmap(infopix);
  infoBtn->resize(infopix.width()+6, infopix.height()+6);
  connect(infoBtn, SIGNAL(clicked()), SLOT(infoRequest()));
  editBox = new QMultiLineEdit(this);
  editLabel = new QLabel(editBox, i18n("Enter Chat Description:"), this);
  updateAnim = new QLabel(this);
  QString filename = locate("kicq_pics", "search.gif");
  QFile testfile(filename);
  if(!testfile.exists())
    filename = locate("kicq_pics", "search.gif");
  updateAnim->setMovie(QMovie((const QString &)filename));
  updateAnim->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  updateAnim->setAlignment(AlignCenter);
  updateAnim->movie()->pause();
  setName(n);
  setUIN(u);
  setEMail(em);
  editBox->setFocus();
  resize(350, 250);
  setMinimumSize(350, 250);
  return;
}

ChatSndWindow::~ChatSndWindow(void)
{
  RemovePointer(updateAnim);
  RemovePointer(editLabel);
  RemovePointer(editBox);
  RemovePointer(infoBtn);
  RemovePointer(emailLabel);
  RemovePointer(emailLine);
  RemovePointer(handleLabel);
  RemovePointer(handleLine);
  RemovePointer(uinLabel);
  RemovePointer(uinLine);
  RemovePointer(cancelBtn);
  RemovePointer(sendBtn);
  RemovePointer(toBox);
  return;
}

void ChatSndWindow::CrashGuard(void)
{
  toBox = 0;
  sendBtn = 0;
  cancelBtn = 0;
  uinLine = 0;
  uinLabel = 0;
  handleLine = 0;
  handleLabel = 0;
  emailLine = 0;
  emailLabel = 0;
  infoBtn = 0;
  editBox = 0;
  editLabel = 0;
  updateAnim = 0;
  return;
}

void ChatSndWindow::resizeEvent(QResizeEvent *re)
{
  IcqWindow::resizeEvent(re);
  QSize uinLblSize, handleLblSize, emailLblSize, toBoxSize;
  QSize editLblSize, btn1Size, btn2Size;

  toBoxSize.setWidth(width()-6);
  uinLblSize = uinLabel->sizeHint();
  handleLblSize = handleLabel->sizeHint();
  emailLblSize = emailLabel->sizeHint();
  int wl = QMAX(uinLblSize.width(), emailLblSize.width());
  int w = (toBoxSize.width()-uinLblSize.width()-handleLblSize.width()-36)/2;
  uinLine->resize(w, uinLine->sizeHint().height());
  handleLine->resize(w, uinLine->sizeHint().height());
  emailLine->resize(toBoxSize.width()-infoBtn->width()-62, uinLine->sizeHint().height());
  uinLabel->resize(wl, uinLine->height());
  handleLabel->resize(handleLblSize.width(), handleLine->height());
  emailLabel->resize(wl, emailLine->height());
  uinLabel->move(12,15);
  uinLine->move(3+uinLabel->x()+uinLabel->width(), 15);
  handleLabel->move(12+uinLine->x()+uinLine->width(), 15);
  handleLine->move(3+handleLabel->x()+handleLabel->width(), 15);
  emailLabel->move(12, 20+uinLine->height());
  emailLine->move(3+emailLabel->x()+emailLabel->width(), 20+uinLine->height());
  toBox->setGeometry(3, 3, toBoxSize.width(), uinLine->height()+emailLine->height()+23);
  infoBtn->move(7+emailLine->x()+emailLine->width(),
                emailLine->y()+(emailLine->height()-infoBtn->height())/2);
  btn1Size = sendBtn->sizeHint();
  btn2Size = cancelBtn->sizeHint();
  int h = QMAX(btn1Size.height(), btn2Size.height());
  w = QMAX(QMAX(btn1Size.width(), btn2Size.width()), 75);
  cancelBtn->setGeometry(width()/2-w-6, height()-h-12, w, h);
  sendBtn->setGeometry(width()/2+6, height()-h-12, w, h);

  editLblSize = editLabel->sizeHint();
  editLabel->setGeometry(6, 8+toBox->height(), editLblSize.width(), editLblSize.height());
  editBox->setGeometry(3, editLabel->y()+editLabel->height()+4, width()-6,
                       height()-editLabel->y()-editLabel->height()-h-28);
  updateAnim->setGeometry(width()-40, height()-40, 32, 32);
  updateAnim->movie()->restart();
  return;
}

void ChatSndWindow::setUIN(unsigned long u)
{
  uin = u;
  uinLine->setText(QString::number(uin));
  return;
}

void ChatSndWindow::setName(const QString &name)
{
  handleLine->setText(name);
  return;
}

void ChatSndWindow::setEMail(const QString &email)
{
  emailLine->setText(email);
  return;
}

void ChatSndWindow::sendChatReq(void)
{
  QString tmp;
  if(MW->isLogged())
  {
    tmp = editBox->text();
    if(MW->isConvertCRLF())
      tmp.replace(QRegExp("\n"), "\r\n");
    sequence = icq_SendChatRequest(MW->icqlink, uin, editBox->text().latin1());
    updateAnim->movie()->unpause();
    sendBtn->setEnabled(FALSE);
  }
  else
    QMessageBox::warning(this, i18n("Error"), i18n("You are not connected to the icq network!"));
  return;
}

void ChatSndWindow::cancelChatReq(void)
{
  if(sequence)
    stopCommand();
  else
    close();
  return;
}

void ChatSndWindow::infoRequest(void)
{
  MW->infoWindow(uin);
  return;
}

void ChatSndWindow::stopCommand(void)
{
  updateAnim->movie()->pause();
  sendBtn->setEnabled(TRUE);
  sequence = 0;
  return;
}

void ChatSndWindow::saveHistory(void)
{
  time_t curtm = time(0);
  struct tm *t = localtime(&curtm);
  TimeStr time(t->tm_hour, t->tm_min, t->tm_mday, t->tm_mon+1, t->tm_year+1900);

  icqMessage msg(uin, time, editBox->text(), MSG_URL_SENT);
  msg.save();
  MW->updateHistoryWindows(msg.uin());
  return;
}
