/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * $Id: contactitem.cpp,v 1.8 2000/10/14 21:00:50 davidvh Exp $
 * $Log: contactitem.cpp,v $
 * Revision 1.8  2000/10/14 21:00:50  davidvh
 * Minor fixes.
 *
 * Revision 1.7  2000/10/03 19:30:05  kir
 * Some UConfig typo fixes.
 *
 * Revision 1.6  2000/10/03 16:19:22  napalmbox
 * fix fix ...
 *
 * Revision 1.5  2000/10/02 21:44:06  kir
 * New UConfig code.
 *
 * Revision 1.4  2000/09/30 20:05:40  napalmbox
 * big update
 *
 * Revision 1.3  2000/09/13 19:11:12  napalmbox
 * replase kicq ver 1.0.0 files with kicq ver 2.0.0 files
 *
 * Revision 1.1.2.2  2000/09/02 21:41:12  napalmbox
 * fix fix fix ....a big fix
 *
 * Revision 1.1.2.1  2000/07/22 19:44:03  napalmbox
 * kicq compiles now with -DQT_NO_ASCII_CAST
 *
 * Revision 1.1  2000/06/17 17:38:03  denis
 * Contact List code was rewritten from scratch! Although we are trying
 * to not add new features in anticipation of the next stable release,
 * but this rewrite solves many things, removes many hacks and stabilizes
 * whole program.
 * One of bug, needs to mention, is an absence of IP address in Info
 * window. The root of problem was in Contact List, because in any time
 * there were several ContactItems in memory for single user - they were
 * "smeared" on the whole code. Now it's clean and strict - all Contact
 * List operations can be done only by ContactList class.
 *
 */

#include <icq.h>

#include "mainwindow.h"
#include "uconfigdb.h"
#include "contactitem.h"

ContactItem::ContactItem(unsigned long puin) :
  nick("N/A"), alias(""), firstName("N/A"), lastName("N/A"), email("N/A"),
  lastIp(0), realIp(0), port(0), tcpflag(0), city("N/A"), state("N/A"), zip("N/A"),
  country(0), countryStat(0), homePage("N/A"), age(0), phone("N/A"), gender(0),
  about("N/A"), notes(""), status((unsigned long)STATUS_OFFLINE), auth(0),
  historyDir(""), msglast(LAST_NONE), onlineAlert(0), visibleUser(0), thruServer(0)
{
  uin = puin;
//  bp = MW->contactsPath() + QString::number(puin);
  return;
}

ContactItem::ContactItem(unsigned long puin, QString yes) :
  nick("N/A"), alias(""), firstName("N/A"), lastName("N/A"), email("N/A"),
  lastIp(0), realIp(0), port(0), tcpflag(0), city("N/A"), state("N/A"), zip("N/A"),
  country(0), countryStat(0), homePage("N/A"), age(0), phone("N/A"), gender(0),
  about("N/A"), notes(""), status((unsigned long)STATUS_OFFLINE), auth(0),
  historyDir(""), msglast(LAST_NONE), onlineAlert(0), visibleUser(0), thruServer(0)
{
//  uin = puin
//  bp = MW->contactsPath() + QString::number(puin);
  load(puin, yes);
  return;
}

ContactItem::ContactItem(unsigned long puin, QString pnick, QString pfirst,
                         QString plast, QString pemail, unsigned char pauth) :
  alias(""), lastIp(0), realIp(0), port(0), tcpflag(0), city("N/A"), state("N/A"),
  zip("N/A"), country(0), countryStat(0), homePage("N/A"), age(0), phone("N/A"),
  gender(0), about("N/A"), notes(""), status((unsigned long)STATUS_OFFLINE),
  historyDir(""), msglast(LAST_NONE), onlineAlert(0), visibleUser(0), thruServer(0)
{
  uin = puin;
  nick = pnick;
  firstName = pfirst;
  lastName = plast;
  email = pemail;
  auth = pauth;
//  bp = MW->contactsPath() + QString::number(puin);
  return;
}

ContactItem::~ContactItem(void)
{
  messageList.clear();
  return;
}

void ContactItem::load(unsigned long puin, QString yes)
{
  uin = puin;

  bp = MW->getOwnerPath() + "/Contact List/" + QString::number(uin);

  db->setPath(bp + "/Info/Main");

  nick      = db->readEntry("Nick").toString();
  email     = db->readEntry("EMail").toString();
  firstName = db->readEntry("FirstName").toString();
  lastName  = db->readEntry("LastName").toString();
  alias     = db->readEntry("Alias").toString();

  if(alias.isEmpty())
    alias = calcAlias();

    // Read home info.
  db->setPath(bp + "/Info/Home");

  city        = db->readEntry("City").toString();
  state       = db->readEntry("State").toString();
  zip         = db->readEntry("Zip").toString();
  country     = db->readEntry("Country").toInt();
  countryStat = db->readEntry("CountryStatus").toInt();
  phone       = db->readEntry("Phone").toString();

  // Read 'more' info.
  db->setPath(bp + "/Info/More");
  homePage = db->readEntry("HomePage").toString();
  age      = db->readEntry("Age").toInt();
  gender   = db->readEntry("Gender").toInt();

  // Read about info
  db->setPath(bp + "/Info/About");
  about = db->readEntry("About").toString();
  notes = db->readEntry("AboutNotes").toString();

  // Read Status
  db->setPath(bp + "/Status");
  status = (unsigned long)STATUS_OFFLINE;
  onlineAlert = db->readEntry("OnlineAlert").toInt();
  visibleUser = db->readEntry("VisibleUser").toInt();
  return;
}

void ContactItem::save(void)
{
  bp = MW->getOwnerPath() + "/Contact List/" + QString::number(uin);

  db->setPath(bp + "/Info/Main");
  db->writeEntry("Nick", nick);
  db->writeEntry("Alias", alias);
  db->writeEntry("FirstName", firstName);
  db->writeEntry("LastName", lastName);
  db->writeEntry("EMail", email);

  // Write Home info
  db->setPath(bp + "/Info/Home");
  db->writeEntry("City", city);
  db->writeEntry("State", state);
  db->writeEntry("Zip", zip);
  db->writeEntry("Country", (unsigned int)country);
  db->writeEntry("CountryStatus", countryStat);
  db->writeEntry("Phone", phone);

  // Write 'More' info
  db->setPath(bp + "/Info/More");
  db->writeEntry("HomePage", homePage);
  db->writeEntry("Age", (unsigned int)age);
  db->writeEntry("Gender", (int)gender);

  // About
  db->setPath(bp + "/Info/About");
  db->writeEntry("About", about);
  db->writeEntry("AboutNotes", notes);

  // Status
  db->setPath(bp + "/Status");
  int z=0; // Needed by QVariant for compiler compatibility.
  db->writeEntry("OnlineAlert", QVariant(onlineAlert, z));
  db->writeEntry("VisibleUser", QVariant(visibleUser, z));
  db->flush(true);
  return;
}

void ContactItem::remove(void)
{
  db->setPath(MW->getOwnerPath() + "/Contact List/" + QString::number(uin));
//  db->setPath(bp);
  db->deleteNode(true);
  db->flush();
  return;
}

icqMessage *ContactItem::getMsg(int n)
{
  return messageList.at(n);
}

int ContactItem::getNumMsgs(void)
{
  return messageList.count();
}

void ContactItem::setMsgLast(lastEvent l)
{
  msglast = l;
  return;
}

void ContactItem::addMsg(icqMessage *msg)
{
  messageList.append(msg);
  if(msg->type() == MSG_RECV)
    setMsgLast(LAST_MSG);
  else if(msg->type() == MSG_URL_RECV)
    setMsgLast(LAST_URL);
  else if(msg->type() == MSG_AUTH_RECV)
    setMsgLast(LAST_AUTH);
  else if(msg->type() == MSG_CHAT_REQ_RECV)
    setMsgLast(LAST_MSG);
  else if(msg->type() == MSG_ADDED)
    setMsgLast(LAST_EXCLAM);
  else if(msg->type() == MSG_FILE_REQ_RECV)
    setMsgLast(LAST_MSG);
  msg->save();
  MW->updateHistoryWindows(msg->uin());
  return;
}

icqMsgType ContactItem::getFirstUnreadMsgType(void)
{
  icqMessage *msg;
  for(msg = messageList.first(); msg; msg = messageList.next())
    if(msg->isNew())
      return msg->type();
  return MSG_UNDEF;
}

int ContactItem::getNumUnreadMsgs(void)
{
  icqMessage *msg;
  int msgCount = 0;
  for(msg = messageList.first(); msg; msg = messageList.next())
    if(msg->isNew())
      msgCount++;
  return msgCount;
}

void ContactItem::updateInfo(QString pnick, QString pfirst, QString plast, QString pemail, char pauth)
{
  nick = pnick;
  firstName = pfirst;
  lastName = plast;
  email = pemail;
  auth = pauth;
  if(alias.isEmpty())
    alias = calcAlias();
  return;
}

void ContactItem::updateExtInfo(QString pcity, unsigned short pcountry_code,
                                char pcountry_stat, QString pstate, unsigned short page,
                                char pgender, QString pphone, QString php, QString pabout)
{
  city = pcity;
  state = pstate;
  country = pcountry_code;
  countryStat = pcountry_stat;
  homePage = php;
  age = page;
  phone = pphone;
  gender = pgender;
  about = pabout;
  return;
}

QString ContactItem::calcAlias(void) const
{
  if(!nick.isEmpty() && nick!="N/A")
    return nick;
  else if(!firstName.isEmpty() && firstName!="N/A")
    return firstName;
  else if(!lastName.isEmpty() && lastName!="N/A")
    return lastName;
  else if(!email.isEmpty() && email!="N/A")
    return email;
  else
    return QString::number(uin);
}

void ContactItem::setStatus(unsigned long pstatus)
{
  status = pstatus;
  return;
}

QString ContactItem::getName(void) const
{
  return (alias.isEmpty()) ? calcAlias() : alias;
}

void ContactItem::setLastIP(unsigned long ip)
{
  lastIp = ip;
  return;
}

void ContactItem::setRealIP(unsigned long ip)
{
  realIp = ip;
  return;
}

void ContactItem::setPort(unsigned short p)
{
  port = p;
  return;
}

void ContactItem::setTCPFlag(unsigned short t)
{
  tcpflag = t;
  return;
}

void ContactItem::setAlias(QString palias)
{
  alias = palias;
  return;
}

void ContactItem::setNotes(QString pnotes)
{
  notes = pnotes;
  return;
}

void ContactItem::setOnlineAlert(bool oa)
{
  onlineAlert = oa;
  return;
}

void ContactItem::setVisible(bool vu)
{
  visibleUser = vu;
  return;
}

void ContactItem::setThruServer(bool ts)
{
  thruServer = ts;
  return;
}
