/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: systembar.cpp,v 1.7 2001/02/22 05:56:01 bills Exp $
// $Log: systembar.cpp,v $
// Revision 1.7  2001/02/22 05:56:01  bills
// fix crash on exit
//
// Revision 1.6  2000/10/14 21:00:50  davidvh
// Minor fixes.
//
// Revision 1.5  2000/09/30 20:05:40  napalmbox
// big update
//
// Revision 1.4  2000/09/15 19:28:29  napalmbox
// add mising header
//
// Revision 1.3  2000/09/14 01:04:34  napalmbox
// sound fixed
//
// Revision 1.2  2000/09/13 19:11:12  napalmbox
// replase kicq ver 1.0.0 files with kicq ver 2.0.0 files
//
// Revision 1.1.2.2  2000/07/08 19:35:33  napalmbox
// fix ..fix ...
//
// Revision 1.1.2.1  2000/07/02 05:45:54  napalmbox
// port kicq -> kde2 :))
//
// Revision 1.1  2000/06/17 17:38:03  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.1  2000/05/24 16:59:50  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.9  2000/01/10 16:09:23  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.8  1999/09/29 17:35:37  denis
// Cleanups.
//
// Revision 1.7  1999/07/16 15:48:58  denis
// Cleaned up.
//
// Revision 1.6  1999/07/16 13:12:26  denis
// Internalization support added.
//
// Revision 1.5  1999/07/12 15:23:32  cproch
// - changed icq_* function calls and callback function to support
//   icqlib with ICQLINK identifier.
// - look for FIXME in mainwindow.cpp
//
// Revision 1.4  1998/12/11 18:29:29  whale
// Changed inheritance to WBlinkListBox to make system bar able to blink
//
// Revision 1.3  1998/12/08 16:31:00  denis
// Focus code removed
// Cleaned up a little before releasing
//

#include <stdlib.h>
#include <qpopmenu.h>
#include <qpainter.h>
#include <kiconloader.h>
#include <kapp.h>
#include "main.h"
#include "systemitem.h"
#include "systembar.h"
#include "icqmessage.h"

#include "systembar.moc"

SystemBar::SystemBar(QWidget *parent, const char *name, WFlags f) : QListBox(parent, name, f)
{
  CrashGuard();
  QPalette pal = palette();
  QColorGroup colgrp = pal.inactive();
  pal.setInactive(QColorGroup(colgrp.foreground(), colgrp.background(),
                              colgrp.light(), colgrp.dark(), colgrp.mid(),
                              colgrp.text(), colgrp.background()));
  colgrp = pal.active();
  pal.setActive(QColorGroup(colgrp.foreground(), colgrp.background(),
                            colgrp.light(), colgrp.dark(), colgrp.mid(),
                            colgrp.text(), colgrp.background()));

  setPalette(pal);

  useritem = new SystemItem();
  insertItem(useritem);

  popup = new QPopupMenu();
  popup->insertItem(UserIcon("history"), i18n("History and OutBox"), 0/*this, SLOT(openSystemHistory())*/);
  popup->setItemEnabled(0, FALSE);
  popup->insertItem(UserIcon("file"), i18n("Incoming Files Folder"), this, SLOT(openIncomingFolder()));
  popup->insertItem(UserIcon("bookmarks"), i18n("Incoming Bookmarks"), 1/*this, SLOT(openIncomingBookmarks())*/);
  popup->setItemEnabled(1, FALSE);
  return;
}

SystemBar::~SystemBar(void)
{
  clear();
  RemovePopup(popup);
  return;
}

void SystemBar::CrashGuard(void)
{
  useritem = 0;
  popup = 0;
  return;
}

void SystemBar::paintCell(QPainter *p, int row, int)
{
  SystemItem *lbi = (SystemItem*)item(row);
  if(!lbi)
    return;

  QColorGroup g = colorGroup();
  {
    p->setBackgroundColor(g.base());
    p->setPen(g.text());
  }
  lbi->paint(p);
  p->setBackgroundColor(g.base());
  p->setPen(g.text());
  return;
}

void SystemBar::mousePressEvent(QMouseEvent *e)
{
  QListBox::mousePressEvent(e);
  if(e->button()==RightButton)
    popup->popup(mapToGlobal(QPoint(e->x(), e->y())));
  return;
}

void SystemBar::openSystemHistory(void)
{
  return;
}

void SystemBar::openIncomingFolder(void)
{
  QString in = locateLocal("appdata", "ReceivedFiles/");
  system(QString("kfmclient openURL '" + in + "' &").latin1());
  return;
}

void SystemBar::openIncomingBookmarks(void)
{
  return;
}
