/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: filesndwin.cpp,v 1.10 2001/01/17 02:01:54 bills Exp $
// $Log: filesndwin.cpp,v $
// Revision 1.10  2001/01/17 02:01:54  bills
// port kicq to new chat/file session interface, add icqlib subdir
//
// Revision 1.9  2000/12/19 06:17:06  bills
// change icqlink member in regwin and mainwindow to an ICQLINK *, use
// icq_ICQLINKNew and icq_ICQLINKDelete instead of icq_Init and icq_Done
//
// Revision 1.8  2000/10/14 21:00:50  davidvh
// Minor fixes.
//
// Revision 1.7  2000/09/30 20:05:40  napalmbox
// big update
//
// Revision 1.6  2000/09/14 02:24:59  napalmbox
// filetransfer fixed
//
// Revision 1.5  2000/09/13 19:11:12  napalmbox
// replase kicq ver 1.0.0 files with kicq ver 2.0.0 files
//
// Revision 1.3.2.2  2000/07/22 19:47:59  napalmbox
// kicq now compiles with -DQT_NO_ASCII_CAST
//
// Revision 1.3.2.1  2000/07/02 05:45:55  napalmbox
// port kicq -> kde2 :))
//
// Revision 1.3  2000/06/17 17:38:34  denis
// Contact List code was rewritten from scratch! Although we are trying
// to not add new features in anticipation of the next stable release,
// but this rewrite solves many things, removes many hacks and stabilizes
// whole program.
// One of bug, needs to mention, is an absence of IP address in Info
// window. The root of problem was in Contact List, because in any time
// there were several ContactItems in memory for single user - they were
// "smeared" on the whole code. Now it's clean and strict - all Contact
// List operations can be done only by ContactList class.
//
// Revision 1.2  2000/06/15 01:58:00  bills
// fixed crash bug when cancelling a file request and then trying to send
// again, now calls icq_CancelFileRequest when request is cancelled to notify
// remote icq client
//
// Revision 1.1  2000/05/24 16:57:49  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.3  2000/05/05 03:38:06  bills
// removed unnecessary deletes in destructor, removed debug statements,
// disabled send button until files are selected.
//
// Revision 1.2  2000/05/04 16:20:28  bills
// updated to use new icqlib file transfer notification method, which enables
// support for sending files.
//
// Revision 1.1  2000/01/20 20:03:32  bills
// added
//
//

#include <stdlib.h>
#include <icq.h>
#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include <kapp.h>
#include <kiconloader.h>
#include <kfiledialog.h>

#include "filewindow.h"
#include "filesndwin.h"
#include "mainwindow.h"
#include "userlist.h"
#include "icqmessage.h"

#include "filesndwin.moc"

FileSndWindow::FileSndWindow(const QString &n, unsigned long u, const QString &em) : IcqWindow()
{
  CrashGuard();
  setIcon(UserIcon("file"));
  QString text;
  QPixmap infoPix;
  setCaption(i18n("Send File Request"));
  toBox = new QGroupBox(i18n("To"), this);
  sendBtn = new QPushButton(i18n("&Send"), this);
//  sendBtn->setAccel(Key_Return+CTRL);
  sendBtn->setEnabled(false);
  connect(sendBtn, SIGNAL(clicked()), SLOT(sendFileReq()));
  cancelBtn = new QPushButton(i18n("&Cancel"), this);
  connect(cancelBtn, SIGNAL(clicked()), SLOT(cancelFileReq()));
  selectBtn = new QPushButton(i18n("Select &Files..."), this);
  connect(selectBtn, SIGNAL(clicked()), SLOT(selectFileReq()));
  uinLine = new QLineEdit(this);
  uinLine->setFocusPolicy(QWidget::NoFocus);
  uinLabel = new QLabel(uinLine, i18n("ICQ #:"), this);
  handleLine = new QLineEdit(this);
  handleLine->setFocusPolicy(QWidget::NoFocus);
  handleLabel = new QLabel(handleLine, i18n("Handle:"), this);
  emailLine = new QLineEdit(this);
  emailLine->setFocusPolicy(QWidget::NoFocus);
  emailLabel = new QLabel(emailLine, i18n("E-Mail:"), this);
  infoBtn = new QPushButton(this);
  infoPix = UserIcon("info2");
  infoBtn->setPixmap(infoPix);
  infoBtn->resize(infoPix.width()+6, infoPix.height()+6);
  connect(infoBtn, SIGNAL(clicked()), SLOT(infoRequest()));
  editBox = new QMultiLineEdit(this);
  editLabel = new QLabel(editBox, i18n("Enter File Description:"), this);
  updateAnim = new QLabel(this);
  QString filename = locate("kicq_pics", "search.gif");
  QFile testfile(filename);
  if(!testfile.exists())
    filename = locate("kicq_pics", "search.gif");
  updateAnim->setMovie(QMovie((const QString &)filename));
  updateAnim->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  updateAnim->setAlignment(AlignCenter);
  updateAnim->movie()->pause();
  setName(n);
  setUIN(u);
  setEMail(em);
  files = 0;
  editBox->setFocus();
  setMinimumSize(400, 250);
  resize(420, 250);
  return;
}

FileSndWindow::~FileSndWindow(void)
{
  RemovePointer(updateAnim);
  RemovePointer(editLabel);
  RemovePointer(editBox);
  RemovePointer(infoBtn);
  RemovePointer(emailLabel);
  RemovePointer(emailLine);
  RemovePointer(handleLabel);
  RemovePointer(handleLine);
  RemovePointer(uinLabel);
  RemovePointer(uinLine);
  RemovePointer(selectBtn);
  RemovePointer(cancelBtn);
  RemovePointer(sendBtn);
  RemovePointer(toBox);
  return;
}

void FileSndWindow::CrashGuard(void)
{
  toBox = 0;
  sendBtn = 0;
  cancelBtn = 0;
  selectBtn = 0;
  uinLine = 0;
  uinLabel = 0;
  handleLine = 0;
  handleLabel = 0;
  emailLine = 0;
  emailLabel = 0;
  infoBtn = 0;
  editBox = 0;
  editLabel = 0;
  updateAnim = 0;
  return;
}

void FileSndWindow::resizeEvent(QResizeEvent *re)
{
  IcqWindow::resizeEvent(re);
  QSize uinLblSize, handleLblSize, emailLblSize;
  QSize toBoxSize, editLblSize, btn1Size, btn2Size;
  toBoxSize.setWidth(width()-6);
  uinLblSize = uinLabel->sizeHint();
  handleLblSize = handleLabel->sizeHint();
  emailLblSize = emailLabel->sizeHint();
  int wl = QMAX(uinLblSize.width(), emailLblSize.width());
  int w = (toBoxSize.width()-uinLblSize.width()-handleLblSize.width()-36)/2;
  uinLine->resize(w, uinLine->sizeHint().height());
  handleLine->resize(w, uinLine->sizeHint().height());
  emailLine->resize(toBoxSize.width()-infoBtn->width()-62,uinLine->sizeHint().height());
  uinLabel->resize(wl, uinLine->height());
  handleLabel->resize(handleLblSize.width(),handleLine->height());
  emailLabel->resize(wl, emailLine->height());
  uinLabel->move(12, 15);
  uinLine->move(3+uinLabel->x()+uinLabel->width(), 15);
  handleLabel->move(12+uinLine->x()+uinLine->width(), 15);
  handleLine->move(3+handleLabel->x()+handleLabel->width(), 15);
  emailLabel->move(12, 20+uinLine->height());
  emailLine->move(3+emailLabel->x()+emailLabel->width(), 20+uinLine->height());
  toBox->setGeometry(3, 3, toBoxSize.width(), uinLine->height()+emailLine->height()+23);
  infoBtn->move(7+emailLine->x()+emailLine->width(),
                emailLine->y()+(emailLine->height()-infoBtn->height())/2);
  btn1Size = sendBtn->sizeHint();
  btn2Size = cancelBtn->sizeHint();
  QSize btn3Size = selectBtn->sizeHint();
  int h = QMAX(QMAX(btn1Size.height(), btn2Size.height()), btn3Size.height());
  w = QMAX(QMAX(QMAX(btn1Size.width(), btn2Size.width()), btn3Size.width()),
      75);
  selectBtn->setGeometry(3, height()-h-12, w, h);
//  selectBtn->setGeometry(width()/3-w-6, height()-h-12, w, h);
  cancelBtn->setGeometry(width()/3+6, height()-h-12, w, h);
  sendBtn->setGeometry(width()/3+w+6, height()-h-12, w, h);

  editLblSize = editLabel->sizeHint();
  editLabel->setGeometry(6, 8+toBox->height(), editLblSize.width(), editLblSize.height());
  editBox->setGeometry(3, editLabel->y()+editLabel->height()+4, width()-6,
                       height()-editLabel->y()-editLabel->height()-h-28);
  updateAnim->setGeometry(width()-40, height()-40, 32, 32);
  updateAnim->movie()->restart();
  return;
}

void FileSndWindow::setUIN(unsigned long u)
{
  uin = u;
  uinLine->setText(QString::number(u));
  return;
}

void FileSndWindow::setName(const QString &name)
{
  handleLine->setText(name);
  return;
}

void FileSndWindow::setEMail(const QString &email)
{
  emailLine->setText(email);
  return;
}

void FileSndWindow::selectFileReq(void)
{
  QString open_filename = KFileDialog::getOpenFileName();

  if(!open_filename.isEmpty())
  {
    if(files)
    {
      if(files[0])
      {
        free(files[0]);
        files[0] = 0;
      }
      free(files);
      files = 0;
    }
    files=(char **)malloc(sizeof(char *)*2);
    files[0]=(char *)malloc(open_filename.length()+1);
    files[1] = 0;
    strcpy(files[0], open_filename.latin1());
    sendBtn->setEnabled(true);
  }
  return;
}

void FileSndWindow::sendFileReq(void)
{
  QString tmp;
  if(MW->isLogged())
  {
    tmp = editBox->text();
    if(MW->isConvertCRLF())
      tmp.replace(QRegExp("\n"),"\r\n");
    sequence = icq_SendFileRequest(MW->icqlink, uin, editBox->text().latin1(), files);
    if(files)
    {
      if(files[0])
      {
        free(files[0]);
        files[0] = 0;
      }
      free(files);
      files = 0;
    }
    updateAnim->movie()->unpause();
    sendBtn->setEnabled(FALSE);
  }
  else
    QMessageBox::warning(this, i18n("Error"), i18n("You are not connected to the icq network!"));
  return;
}

void FileSndWindow::cancelFileReq(void)
{
  if(sequence)
    stopCommand();
  else
    close();
  return;
}

void FileSndWindow::infoRequest(void)
{
  MW->infoWindow(uin);
  return;
}

void FileSndWindow::stopCommand(void)
{
  if(sequence)
  {
    icq_CancelFileRequest(MW->icqlink, uin, sequence);
    sequence = 0;
  }
  updateAnim->movie()->pause();
  return;
}

void FileSndWindow::saveHistory(void)
{
  time_t curtm = time(0L);
  struct tm *t = localtime(&curtm);
  TimeStr time(t->tm_hour, t->tm_min, t->tm_mday, t->tm_mon+1, t->tm_year+1900) ;

  icqMessage msg(uin, time, editBox->text(), MSG_URL_SENT) ;
  msg.save() ;
  MW->updateHistoryWindows(msg.uin()) ;
  return;
}
