/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
// $Id: filewindow.cpp,v 1.8 2001/05/23 23:54:54 denis Exp $
// $Log: filewindow.cpp,v $
// Revision 1.8  2001/05/23 23:54:54  denis
// Warning cleanup.
//
// Revision 1.7  2001/01/17 02:01:54  bills
// port kicq to new chat/file session interface, add icqlib subdir
//
// Revision 1.6  2000/10/14 21:00:50  davidvh
// Minor fixes.
//
// Revision 1.5  2000/10/07 22:53:04  napalmbox
// clean up layout->activate() and "magic" resize(1,1) - not need for QT2.2.X up
//
// Revision 1.4  2000/09/13 19:11:12  napalmbox
// replase kicq ver 1.0.0 files with kicq ver 2.0.0 files
//
// Revision 1.2.2.1  2000/09/02 21:41:12  napalmbox
// fix fix fix ....a big fix
//
// Revision 1.3  2000/07/30 18:53:25  denis
// Warnings were fixed.
//
// Revision 1.2  2000/06/17 17:04:19  denis
// Cleanups.
//
// Revision 1.1  2000/05/24 16:57:49  denis
// Sources have been splitted to the whole bunch of appropriate
// directories. Every directory is end up with static library. All
// of them are linked together to the final kicq binary.
//
// Revision 1.7  2000/05/04 16:15:00  bills
// updated to use new icqlib file transfer notification method, tweaked
// layout, implemented abort button and timers.
//
// Revision 1.6  2000/03/30 15:13:11  denis
// Float/integer warning fixed.
//
// Revision 1.5  2000/01/20 20:03:03  bills
// initial implementation for sending file requests
//
// Revision 1.4  2000/01/16 21:36:51  bills
// implemented save/save as action buttons, changed code to use new file
// session interface
//
// Revision 1.3  2000/01/10 16:08:14  denis
// Headers reorganized for simplify dependencies and speedup compilation.
//
// Revision 1.2  2000/01/07 04:14:56  bills
// fixed warning with reduceNumber
//
// Revision 1.1  1999/12/28 01:55:27  bills
// added
//

#include <qmsgbox.h>
#include <qmovie.h>
#include <qregexp.h>
#include <qkeycode.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <kapp.h>
#include <kiconloader.h>
#include <qpushbutton.h>

#include "filewindow.h"
#include "mainwindow.h"

#include "filewindow.moc"

QString reduceNumber(float num)
{
  if(num > 2097152)
    return QString().sprintf("%.2f MB", num / 1048576);
  else if(num > 10240) 
    return QString().sprintf("%.2f KB", num / 1024);
  else
    return QString().sprintf("%d bytes", (int)num);
}
  
FileWindow::FileWindow(IcqFileSession *session) :
  IcqWindow(), icqFileSession(session)
{
  CrashGuard();

  mainLayout = new QGridLayout(this, 5, 1, 7);
  mainLayout->setRowStretch(0, 6);
  mainLayout->setRowStretch(1, 0);
  mainLayout->setRowStretch(2, 9);
  mainLayout->setRowStretch(3, 0);
  mainLayout->setRowStretch(4, 3);

  // Header

  headerLayout = new QGridLayout(2, 4, 7);
  mainLayout->addLayout(headerLayout, 0, 0);
  headerLayout->setColStretch(0, 0);
  headerLayout->setColStretch(1, 1);
  headerLayout->setColStretch(2, 0);
  headerLayout->setColStretch(3, 1);

  fileLabel = new QLabel("Filename:", this);
  fileLabel->setMinimumSize(fileLabel->sizeHint());
  headerLayout->addWidget(fileLabel, 0, 0);

  headerFileName = new QLineEdit(this);
  headerFileName->setFocusPolicy(QWidget::NoFocus);
  headerFileName->setMinimumSize(headerFileName->sizeHint());
  headerFileName->setMaximumHeight(headerFileName->sizeHint().height());
  headerLayout->addWidget(headerFileName, 0, 1);

  fileCountLabel = new QLabel("Files:", this);
  fileCountLabel->setMinimumSize(fileCountLabel->sizeHint());
  headerLayout->addWidget(fileCountLabel, 0, 2);

  fileCount = new QLineEdit(this);
  fileCount->setFocusPolicy(QWidget::NoFocus);
  fileCount->setMinimumSize(fileCount->sizeHint());
  fileCount->setMaximumHeight(fileCount->sizeHint().height());
  headerLayout->addWidget(fileCount, 0, 3);

  operationLabel = new QLabel( 
    icqFileSession->isSending() ? "Reading:" : "Saving To:", this);
  operationLabel->setMinimumSize(operationLabel->sizeHint());
  headerLayout->addWidget(operationLabel, 1, 0);

  operationFileName = new QLineEdit(this);
  operationFileName->setFocusPolicy(QWidget::NoFocus);
  operationFileName->setMinimumSize(operationFileName->sizeHint());
  operationFileName->setMaximumHeight(operationFileName->sizeHint().height());
  headerLayout->addWidget(operationFileName, 1, 1);

  statusLabel = new QLabel("Status:", this);
  statusLabel->setMinimumSize(statusLabel->sizeHint());
  headerLayout->addWidget(statusLabel, 1, 2);
  
  statusText = new QLineEdit(this);
  statusText->setFocusPolicy(QWidget::NoFocus);
  statusText->setMinimumSize(statusText->sizeHint());
  operationFileName->setMaximumHeight(statusText->sizeHint().height());
  headerLayout->addWidget(statusText, 1, 3);

  // Bar

  line1 = new QFrame(this);
  line1->setFrameStyle(QFrame::HLine | QFrame::Sunken);
  line1->setLineWidth(1);
  mainLayout->addWidget(line1, 1, 0);

  // Progress section

  midLayout = new QGridLayout(3, 6, 7);
  mainLayout->addLayout(midLayout, 2, 0);
  midLayout->setColStretch(0, 0);
  midLayout->setColStretch(1, 1);
  midLayout->setColStretch(2, 0);
  midLayout->setColStretch(3, 1);
  midLayout->setColStretch(4, 0);
  midLayout->setColStretch(5, 3);

  progressLabel = new QLabel("File:", this);
  progressLabel->setMinimumSize(progressLabel->sizeHint());
  midLayout->addWidget(progressLabel, 0, 0);

  progressBar = new QProgressBar(this);
  //progressBar->setMinimumSize(progressBar->sizeHint());
  midLayout->addMultiCellWidget(progressBar, 0, 0, 1, 3);

  sizeLabel1 = new QLabel("Size:", this);
  sizeLabel1->setMinimumSize(sizeLabel1->sizeHint());
  midLayout->addWidget(sizeLabel1, 0, 4);

  fileSizeText = new QLineEdit(this);
  fileSizeText->setFocusPolicy(QWidget::NoFocus);
  int width=fontMetrics().width("1000.00 MB of 1000.00 MB   ");
  fileSizeText->setMinimumSize(width,fileSizeText->sizeHint().height());
  fileSizeText->setMaximumHeight(fileSizeText->sizeHint().height());
  midLayout->addWidget(fileSizeText, 0, 5);

  progressLabel2 = new QLabel("Batch:", this);
  progressLabel2->setMinimumSize(progressLabel2->sizeHint());
  midLayout->addWidget(progressLabel2, 1, 0);

  batchProgressBar = new QProgressBar(this);
  //batchProgressBar->setMinimumSize(batchProgressBar->sizeHint());
  midLayout->addMultiCellWidget(batchProgressBar, 1, 1, 1, 3);

  sizeLabel2 = new QLabel("Size:", this);
  sizeLabel2->setMinimumSize(sizeLabel2->sizeHint());
  midLayout->addWidget(sizeLabel2, 1, 4);

  batchSizeText = new QLineEdit(this);
  batchSizeText->setFocusPolicy(QWidget::NoFocus);
  width=fontMetrics().width("1000.00 MB of 1000.00 MB   ");
  batchSizeText->setMinimumSize(width,batchSizeText->sizeHint().height());
  batchSizeText->setMaximumHeight(batchSizeText->sizeHint().height());
  midLayout->addWidget(batchSizeText, 1, 5);

  elapsedLabel = new QLabel("Elapsed:", this);
  elapsedLabel->setMinimumSize(elapsedLabel->sizeHint());
  midLayout->addWidget(elapsedLabel, 2, 0);

  elapsedText = new QLineEdit(this);
  elapsedText->setFocusPolicy(QWidget::NoFocus);
  elapsedText->setMinimumSize(elapsedText->sizeHint());
  elapsedText->setMaximumHeight(elapsedText->sizeHint().height());
  midLayout->addWidget(elapsedText, 2, 1);

  remainingLabel = new QLabel("Remaining:", this);
  remainingLabel->setMinimumSize(remainingLabel->sizeHint());
  midLayout->addWidget(remainingLabel, 2, 2);

  remainingText = new QLineEdit(this);
  remainingText->setFocusPolicy(QWidget::NoFocus);
  remainingText->setMinimumSize(remainingText->sizeHint());
  remainingText->setMaximumHeight(remainingText->sizeHint().height());
  midLayout->addWidget(remainingText, 2, 3);

  cpsLabel = new QLabel("CPS:", this);
  cpsLabel->setMinimumSize(cpsLabel->sizeHint());
  midLayout->addWidget(cpsLabel, 2, 4);

  cpsText = new QLineEdit(this);
  cpsText->setFocusPolicy(QWidget::NoFocus);
  cpsText->setMinimumSize(cpsText->sizeHint());
  cpsText->setMaximumHeight(cpsText->sizeHint().height());
  midLayout->addWidget(cpsText, 2, 5);

  // Bar

  QFrame *line2 = new QFrame(this);
  line2->setFrameStyle(QFrame::HLine | QFrame::Sunken);
  line2->setLineWidth(1);
  mainLayout->addWidget(line2, 3, 0);

  // Bottom

  bottomLayout = new QGridLayout(1, 4);
  mainLayout->addLayout(bottomLayout, 4, 0);
  bottomLayout->setColStretch(0, 0);
  bottomLayout->setColStretch(1, 1);
  bottomLayout->setColStretch(2, 0);
  bottomLayout->setColStretch(3, 0);
  
  speedLabel = new QLabel("Speed:", this);
  speedLabel->setMinimumSize(speedLabel->sizeHint());
  bottomLayout->addWidget(speedLabel, 0, 0);

  speedSlider = new QSlider(0, 100, 10, 0, QSlider::Horizontal, this);
  speedSlider->setMinimumSize(speedSlider->sizeHint());
  speedSlider->setMaximumHeight(speedSlider->sizeHint().height());
  speedSlider->setTickInterval(10);
  speedSlider->setTickmarks(QSlider::Above);
  connect(speedSlider, SIGNAL(valueChanged(int)),
    SLOT(sliderChanged(int)));
  bottomLayout->addWidget(speedSlider, 0, 1);

  skipBtn = new QPushButton(this);
  skipBtn->setText("&Skip File");
  skipBtn->setMinimumSize(skipBtn->sizeHint());
  bottomLayout->addWidget(skipBtn, 0, 2);
  skipBtn->setEnabled(false);

  abortBtn = new QPushButton(this);
  abortBtn->setText("&Abort");
  abortBtn->setMinimumSize(abortBtn->sizeHint());
  bottomLayout->addWidget(abortBtn, 0, 3);
  connect(abortBtn, SIGNAL(clicked()), SLOT(abortClicked()));

  resize(500, 250);
  setMinimumSize(400, 250);

  connect(icqFileSession,
          SIGNAL(progress(unsigned long, unsigned long)),
          SLOT(progress(unsigned long, unsigned long)));
  connect(icqFileSession,
          SIGNAL(statusChanged(int)),
          SLOT(statusChanged(int)));
  connect(icqFileSession,
          SIGNAL(closed()),
          SLOT(sessionClosed()));
  connect(icqFileSession,
          SIGNAL(speedChanged(int)),
          speedSlider,
          SLOT(setValue(int)));
  return;
}

FileWindow::~FileWindow(void)
{
  RemovePointer(abortBtn);
  RemovePointer(skipBtn);
  RemovePointer(speedSlider);
  RemovePointer(speedLabel);
  RemovePointer(bottomLayout);
  RemovePointer(line2);
  RemovePointer(cpsText);
  RemovePointer(cpsLabel);
  RemovePointer(remainingText);
  RemovePointer(remainingLabel);
  RemovePointer(elapsedText);
  RemovePointer(elapsedLabel);
  RemovePointer(batchSizeText);
  RemovePointer(sizeLabel2);
  RemovePointer(batchProgressBar);
  RemovePointer(progressLabel2);
  RemovePointer(fileSizeText);
  RemovePointer(sizeLabel1);
  RemovePointer(progressBar);
  RemovePointer(progressLabel);
  RemovePointer(midLayout);
  RemovePointer(line1);
  RemovePointer(statusText);
  RemovePointer(statusLabel);
  RemovePointer(operationFileName);
  RemovePointer(operationLabel);
  RemovePointer(fileCount);
  RemovePointer(fileCountLabel);
  RemovePointer(headerFileName);
  RemovePointer(fileLabel);
  RemovePointer(headerLayout);
  RemovePointer(mainLayout);
  return;
}

void FileWindow::CrashGuard(void)
{
  mainLayout = 0;
  // Header
  headerLayout = 0;
  fileLabel = 0;
  headerFileName = 0;
  fileCountLabel = 0;
  fileCount = 0;
  operationLabel = 0;
  operationFileName = 0;
  statusLabel = 0;
  statusText = 0;
  // Bar
  line1 = 0;
  // Progress section
  midLayout = 0;
  progressLabel = 0;
  progressBar = 0;
  sizeLabel1 = 0;
  fileSizeText = 0;
  progressLabel2 = 0;
  batchProgressBar = 0;
  sizeLabel2 = 0;
  batchSizeText = 0;
  elapsedLabel = 0;
  elapsedText = 0;
  remainingLabel = 0;
  remainingText = 0;
  cpsLabel = 0;
  cpsText = 0;
  // Bar
  line2 = 0;
  // Bottom
  bottomLayout = 0;
  speedLabel = 0;
  speedSlider = 0;
  skipBtn = 0;
  abortBtn = 0;
  return;
}

void FileWindow::progress(unsigned long current, unsigned long total)
{
  progressBar->setProgress(current * current_scale);
  batchProgressBar->setProgress(total * total_scale);

  fileSizeText->setText(reduceNumber(current) + " of " +
                        reduceNumber(icqFileSession->currentFileSize()));
  batchSizeText->setText(reduceNumber(total) +  " of " +
                         reduceNumber(icqFileSession->totalSize()));

  // FIXME: Complete restructure needed. This has too much overhead.
  if(!cps_start_time)
  {
    start_bytes = current - 2048;
    cps_start_time = time(0);
    cps_timer = time(0);
  }   
  if(time(0) > cps_timer)
  {
    int seconds = time(0) - cps_start_time;
    cps = (current - start_bytes) / seconds;
    cpsText->setText(reduceNumber(cps) + "/sec");
    cps_timer = time(0);
  }
  return;
}

void FileWindow::statusChanged(int status)
{
  QTimer *timer;

  switch(status) {
    case FILE_STATUS_LISTENING:
      statusText->setText("Waiting for connection...");
      break;
    case FILE_STATUS_CONNECTING:
      statusText->setText("Connecting...");
      break;
    case FILE_STATUS_CONNECTED:
      statusText->setText("Connected.");
      break;
    case FILE_STATUS_INITIALIZING:
      total_scale=PROGRESS_SCALE/icqFileSession->totalSize();
      batchProgressBar->setTotalSteps(PROGRESS_SCALE);
      fileCount->setText(
        QString().sprintf("0/%d", icqFileSession->totalFiles()));
      if(icqFileSession->isReceiving())
         setCaption(QString("kicq - Receiving files from ") + 
           icqFileSession->remoteHandle());
      else
         setCaption(QString("kicq - Sending files to ") +
                    icqFileSession->remoteHandle());

      timer = new QTimer(this);
      connect(timer, SIGNAL(timeout()), SLOT(updateTimes()));
      timer->start(1000); 
      elapsed_time = 0;

      statusText->setText("Waiting for next file...");
      break;
    case FILE_STATUS_NEXT_FILE:
      headerFileName->setText(icqFileSession->currentFile());
      operationFileName->setText(icqFileSession->currentFile());
      fileCount->setText(QString().sprintf("%d/%d",
                         icqFileSession->currentFileNumber(),
                         icqFileSession->totalFiles()));
      current_scale = PROGRESS_SCALE / icqFileSession->currentFileSize();
      progressBar->reset();
      progressBar->setTotalSteps(PROGRESS_SCALE);
      cps_start_time = 0;
      speedSlider->setValue(icqFileSession->speed());
      statusText->setText("Waiting for data...");
      break;
    case FILE_STATUS_SENDING:
      statusText->setText("Sending...");
      break;
    case FILE_STATUS_RECEIVING:
      statusText->setText("Receiving...");
      break;
    default:
      break;
  }
  return;
}

void FileWindow::sessionClosed()
{
  if (!requestComplete())
  {
    m_requestComplete = true;
    close();
  }
}

void FileWindow::closeEvent(QCloseEvent *ce)
{
  bool complete = requestComplete();

  IcqWindow::closeEvent(ce);
 
  if(!complete)
    icqFileSession->close();

  return;
}

void FileWindow::resizeEvent(QResizeEvent *re)
{
  IcqWindow::resizeEvent(re);
  return;
}

void FileWindow::sliderChanged(int new_value)
{
  if(icqFileSession && icqFileSession->isTransferring())
    icqFileSession->setSpeed(new_value);
  return;
}

void FileWindow::abortClicked(void)
{
  int result = QMessageBox::warning(this, "Warning", 
    "This will stop the file transfer.  Are you sure?",
    QMessageBox::Yes | QMessageBox::Default,
    QMessageBox::No | QMessageBox::Escape);

  if(result == QMessageBox::Yes)
    close();
  return;
}

void FileWindow::updateTimes(void)
{
  elapsed_time++;
  elapsedText->setText(formatTime(elapsed_time));

  if(cps) 
  {
    // Broken down for readability
    long remaining = icqFileSession->totalSize();
    remaining -= icqFileSession->totalProgress();
    remaining = (long)(remaining / cps);
    remainingText->setText(formatTime(remaining));
  }
  return;
}

QString FileWindow::formatTime(long seconds)
{
  long minutes = seconds / 60;
  long hours = minutes / 60;

  return QString().sprintf("%02d:%02d:%02d", (int)hours, (int)minutes, (int)(seconds % 60));
}
