/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
#ifndef _MAINWINDOW_
#define _MAINWINDOW_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qlistbox.h>
#include <qwidget.h>
#include <qpushbt.h>
#include <qlayout.h>
#include <qmsgbox.h>
#include <qstring.h>
#include <qptrdict.h>
#include <qtoolbutton.h>
#include <klocale.h>
#include <kstddirs.h>
#include <kglobal.h>
#include <ktmainwindow.h>
#include <qpopupmenu.h>
#include <kmenubar.h>
#include <ktoolbar.h>
#include <icq.h>
#include "alertbox.h"
#include "connectbutton.h"
#include "contactlist.h"
#include "icqwindow.h"
#include "uconfigdb.h"
#include "sessionmanager.h"

#define MENU_SHUTDOWN     1
#define MENU_PREFERENCES  2
#define MENU_SECURITY     3

#define	MENU_COPYUINTOCLIPBOARD 10
#define	MENU_COPYIPTOCLIPBOARD  11

#ifndef RemoveList
#define RemoveList(x) if(x) { x->clear(); delete x; x = 0; }
#define RemovePopup RemoveList
#endif
#ifndef RemovePointer
#define RemovePointer(x) if(x) { delete x; x = 0; }
#endif

class UserList;
class SystemBar;
class ConnectButton;
class Preferences;
class FindWindow;
class DockWidget;
class IcqSound;
class RegWindow;
class IcqAnimButton;
class ITButton;
class MyInfoWin;

class MainWindow : public KTMainWindow {
  Q_OBJECT
public:
  MainWindow(void);
  ~MainWindow(void);

  ICQLINK *icqlink;
  int playWav;
  IcqSound *sound;
  QPopupMenu *statemenu;
  int isLogged(void) { return logged; }
  const QString logFile(void) { return (const QString&)logfile; }
  static void logEvent(ICQLINK*, time_t, unsigned char, const char*);
  void delAlertWin(AlertMsgBox*);
  void infoWindow(ContactItem);
  void infoWindow(unsigned long);
  void updateHistoryWindows(unsigned long);
  void setIcon(QPixmap);
  void togglePixmap(int);
  int isUnreadMsgs(void);
  QPopupMenu *stateMenu(void) { return statemenu; }
  UserList *listWidget(void) { return listwidget; }
  FindWindow *findWindow(void) { return findwin; }
  RegWindow *regWindow(void) { return regwin; }
  ContactList &contactList(void) { return contactlist; }
  int isConvertCRLF(void) { return convertCRLF; }
  int isQuote(void) { return quote; }
  int isCloseRecv(void) { return closeRecv; }
  void updateContact(void);
  void addIcqWindow(IcqWindow*);
  void delIcqWindow(IcqWindow*);
  void saveConfigOwner(const QString &, const QString &);
  void saveConfigOwner(unsigned long, const QString &, const QString &);
  void saveConfigServer(const QString &, unsigned short);
  void saveConfigServer(unsigned long, const QString &, unsigned short);
  void saveConfigProxy(int, const QString &, unsigned short, int, const QString &, const QString &);
  void saveConfigProxy(unsigned long, int, const QString &, unsigned short, int, const QString &, const QString &);
  void saveConfigLog(int, const QString &);
  void saveConfigLog(unsigned long, int, const QString &);
  void saveConfigSound(int, bool clear = FALSE);
  void saveConfigSound(unsigned long, int, bool clear = FALSE);
  void saveConfigOther(int, int, int, int, int);
  void saveConfigOther(unsigned long, int, int, int, int, int);
  void saveConfigAway(int, int, bool, bool);
  void saveConfigAway(unsigned long, int, int, bool, bool);
  void updateBlinking(void);
  void messageRequest(unsigned long, const QString &);
  void updateVisibleList(void);
  void prepareNewUIN(unsigned long);
  QString getOwnerPath(void);
  QString getOwnerPath(unsigned long);
  unsigned long int parseUIN(QString &);

public slots:
  void setStatus(int);
  void stateCommand(void);
  void recvRequest(int);
  void onlineAlert(void);
  void visibleUser(void);
  void messageRequest(void);
  void urlRequest(void);
  void authRequest(void);
  void chatRequest(void);
  void fileRequest(void);
  void historyRequest(void);
  void infoRequest(void);
  void renameRequest(void);
  void delUserRequest(void);
  void readFirstRequest(void);
  void recvRequest(void);
  void mailRequest(QString address);
  void mailRequest(void);
  void specialOpRequest(int);
  void webBrowserRequest(QString url);
  void webBrowserRequest(int userNumber, QString protocol);
  void tetrinetRequest(QString ip);
  void tetrinetRequest(int userNumber);
  void telnetRequest(QString ip);
  void telnetRequest(int userNumber);
  void pingRequest(QString ip);
  void pingRequest(int userNumber);
  void tracerouteRequest(QString ip);
  void tracerouteRequest(int userNumber);
  QString intIP2StringIP(unsigned long);
  virtual void slotHelpAbout(void);
  void homepage(void);
signals:
protected:
  QGridLayout *grid;

  bool autoawayenabled; // is auto-away enabled?
  bool autoawaytona;    // change to NA if user set away?
  bool autoawayset;     // is auto-away active?
  int autoawaytimer;    // timer to cause timerEvent which calls MainWindow::autoAway()
  int autoawaytimeout;  // how long to wait before changing status to away
  int autonatimeout;    // how long to wait before changing status from away to NA
  int logged;
  int keepalivetimer;
  unsigned long uin;
  QString passwd;
  QString nick;
  QString server;
  unsigned short port;
  int useproxy;
  QString proxyhost;
  unsigned short proxyport;
  int proxyauth;
  QString proxyuser, proxypass;
  QString logfile;
  int bell;
  int userPlay;
  int closeRecv;
  int convertCRLF;
  int quote;
  int reconnect;
  icqStatus log_status;
  icqStatus cur_status;

  DockWidget *dockwidget;
  MyInfoWin *myinfowin;
  QWidget *panelwidget;
  UserList *listwidget;
  SystemBar *syswidget;
  IcqAnimButton *menubutton;
  ITButton *addbutton;
  ConnectButton *statebutton;
  QToolButton *allbutton, *groupbutton;
  QSpacerItem *spacer, *spacer_2;
  QLabel *mlabel;
  QPopupMenu *ownersmenu;
  QPopupMenu *myicqmenu, *icqmenu;
  Preferences *prefwin;
  FindWindow *findwin;
  RegWindow *regwin;

  QString appdir, contdir;
  QPixmap blinkPixmap[2];
  QPixmap stateIcon;
  int blinkFlag;
  int blinkPixNum;
  QList<IcqWindow> icqWinList;
  QList<AlertMsgBox> alertWinList;
  ContactList contactlist;
  SessionManager sessionManager;

  void closeEvent(QCloseEvent*);
  void setupGUICallbacks(void);
  virtual void timerEvent(QTimerEvent*);
  void login(unsigned long);
  void disconnect(void);
  bool checkConfig(void);
  void setTitle(void);
  QStrList getUINDirList(void);
  void stopAll(void);
  void doNotify(unsigned long, int, unsigned int, void*);
  void doChatNotify(icq_ChatSession *, int, int, void*);
  void doFileNotify(icq_FileSession *, int, int, void*);
  void updateRecvWindows(void);
  void setKeyboardLED(bool on);
  QString encryptStr(QString aStr);
  QString decryptStr(const QString aStr);
  QString getPassword(void);
  void setPassword(QString password);
  icqStatus loadConfig(void);
  DockWidget *dockWidget(void) { return dockwidget; }
  void saveProperties(void);
  void saveStatus(void);
  void saveProperties(unsigned long);
  void saveStatus(unsigned long);
  void saveUser(unsigned long, QString, QString, QString, QString, unsigned char);
  void initContact(void);
  void addAlertWin(unsigned long);
  void infoReplyHandler(unsigned long, const QString &, const QString &, const QString &, const QString &, char);
  void infoExtReplyHandler(unsigned long, const QString &, unsigned short, char,
                           const QString &, unsigned short, char, const QString &, const QString &, const QString &);
  icqStatus status(void);
  void setBlinking(int);
  unsigned long getFirstUnreadMsgUIN(void);
  void setBlinkPixmap(const QString &);
  void setToolTip(void);
  void autoAway(void);
  void messageRequest(int);
  void chatRequest(int);
  void fileRequest(int);
  void urlRequest(int);
  void authRequest(int);
  void historyRequest(int);
  void setLogStatus(void);
  void setAllOnline(bool);
  bool getAllOnline(void);
  void setOnlineStatus(icqStatus, unsigned long);
  unsigned long getOwnerConfigUIN(void);
  void setOwnerConfigUIN(unsigned long);
  void fillOwnersPopup(void);
  void startOwner(void);
  // Callbacks
  static void loggedIn(ICQLINK*);
  static void disconnected(ICQLINK*);
  static void userOnline(ICQLINK*, unsigned long, unsigned long,
              unsigned long, unsigned short, unsigned long, unsigned char);
  static void userStatusUpdate(ICQLINK*, unsigned long, unsigned long);
  static void userOffline(ICQLINK*, unsigned long);
  static void webPager(ICQLINK*, unsigned char, unsigned char, unsigned char,
              unsigned char, unsigned short, const char*, const char*,
              const char*);
  static void mailExpress(ICQLINK*, unsigned char, unsigned char,
              unsigned char, unsigned char, unsigned short, const char*,
              const char*, const char*);
  static void message(ICQLINK*, unsigned long, unsigned char,
              unsigned char, unsigned char, unsigned char, unsigned short,
              const char*);
  static void url(ICQLINK*, unsigned long, unsigned char, unsigned char,
              unsigned char, unsigned char, unsigned short, const char*,
              const char*);
  static void chatReq(ICQLINK*, unsigned long, unsigned char,
              unsigned char, unsigned char, unsigned char, unsigned short,
              const char*,unsigned long);
  static void fileReq(ICQLINK *link, unsigned long, unsigned char, unsigned char,
              unsigned char, unsigned char, unsigned short, const char*,
              const char *, unsigned long, unsigned long);
  static void AuthRequest(ICQLINK*, unsigned long, unsigned char,
              unsigned char, unsigned char, unsigned char, unsigned short,
              const char*, const char*, const char*, const char*, const char*);
  static void infoReply(ICQLINK*, unsigned long, const char*, const char*,
              const char*, const char*, char);
  static void infoExtReply(ICQLINK*, unsigned long, const char*,
              unsigned short, char, const char*, unsigned short, char,
              const char*, const char*, const char*);
  static void requestNotify(ICQLINK*, unsigned long, int, int, void *);
  static void chatNotify(icq_ChatSession*, int, int, void *);
  static void fileNotify(icq_FileSession*, int, int, void *);
  static void authreq(ICQLINK*, unsigned long, unsigned char,
              unsigned char, unsigned char, unsigned char, unsigned short,
              const char*, const char*, const char*, const char*, const char*);
  static void added(ICQLINK*, unsigned long, unsigned char, unsigned char,
              unsigned char, unsigned char, unsigned short, const char*,
              const char*,const char*, const char*);
  static void setTimeout(ICQLINK*, long);
  ////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // Meta User Info
  ////////////////////////////////////////////////////////////////////////////////////////////////////////////
  static void metaUserInfo(ICQLINK*, unsigned short , const char *, const char *, const char *,
              const char *, const char *, const char *, const char *, const char *, const char *, const char *,
              const char *, const char *, unsigned long, unsigned short , unsigned char , unsigned char ,
              unsigned char , unsigned char);

  static void metaUserWork(ICQLINK*, unsigned short, const char *, const char *, const char *, const char *,
              const char *, unsigned long, unsigned short, const char *, const char *, const char *,
              unsigned short, const char *);
  static void metaUserMore(ICQLINK*, unsigned short, unsigned short , unsigned char, const char *,
              unsigned char, unsigned char, unsigned char, unsigned char, unsigned char, unsigned char);
  static void metaUserAbout(ICQLINK*, unsigned short, const char *);
  static void metaUserInterests(ICQLINK*, unsigned short, unsigned char, unsigned short, const char *,
              unsigned short, const char *, unsigned short, const char *, unsigned short, const char *);
  static void metaUserAffiliations(ICQLINK*, unsigned short, unsigned char, unsigned short, const char *,
              unsigned short, const char *, unsigned short, const char *, unsigned short, const char *,
              unsigned char, unsigned short, const char *, unsigned short, const char *, unsigned short,
              const char *, unsigned short, const char *);
  static void metaUserHomePageCategory(ICQLINK*, unsigned short, unsigned char, unsigned short, const char *);
  ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
protected slots:
  void myDetails(void);
  void quitRequest(void);
  void addCommand(void);
  void allCommand(void);
  void menuCommand(void);
  void icqCommand(int);
  void saveSessionConfig(void);
  void registerCommand(void);
  void debug1Request(void);
  void debug2Request(void);
  void debug3Request(void);
  void debug4Request(void);
  void debug5Request(void);
  void debug6Request(void);
  void debug7Request(void);
  void changeOwner(int);
// FIXME: Put these in the private section.
  void initMainPanel(void);
  void initMainMenus(void);
  void initPreferences(void);
private:
  void CrashGuard(void);
  void clearMainPanel(void);
  void clearMainMenus(void);
  void clearPreferences(void);
  void destroyMainPanel(void);
  void destroyMainMenus(void);
  void destroyPreferences(void);
private slots:
};

extern MainWindow *MW;
extern UConfig    *db;  // Global kicq db.

#endif /* _MAINWINDOW_ */
