/***************************************************************************
    description          : corewidget.h
    begin                : Tue Apr 25 2000
    copyright            : (C) 2000 by Florian Veskov
    email                : napalmbox@icqmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COREWIDGET_H
#define COREWIDGET_H

#include <qlineedit.h>
#include <qpushbt.h>
#include <qlabel.h>
#include <qmlined.h>

class RestrictedLine : public QLineEdit
{
  Q_OBJECT
public:
   RestrictedLine(QWidget *parent=0, const char *name=0, const char *valid=0);
  ~RestrictedLine(void);
  void setValidChars(const char *valid);
  signals:
  void invalidChar(int);
protected:
  void keyPressEvent(QKeyEvent *e);
private:
  QString qsValidChars;
};

enum EditLineType{
  EditTypeOct =  8,
  EditTypeDec = 10,
  EditTypeHex = 16
};

class KIntegerLine : public RestrictedLine
{
  Q_OBJECT
public:
  KIntegerLine(void);
  KIntegerLine(QWidget *parent=0,const char *name=0,EditLineType t=EditTypeDec);
  ~KIntegerLine(void);
  EditLineType getType(void);
  int value(void);
  void setValue(int value);
signals:
  void valueChanged(int);
protected:
  void keyPressEvent(QKeyEvent *e);
private slots:
  void internalValueChanged(void);
private:
  EditLineType lineType;
  int getValue(QString &s);
  void putValue(QString &s, int val);
  void incValue(QString &s, int val);
  void decValue(QString &s, int val);
};

class ULine : public QLineEdit
{
  Q_OBJECT
public:
  ULine(QWidget *parent, bool readonly);
  void SetReadOnly(bool);
  bool ReadOnly(void) { return m_bReadOnly; }
public slots:
  void setEnabled(bool);
protected:
  QColor baseRO, baseRW;
  bool m_bReadOnly;
};

class MLine : public QMultiLineEdit
{
  Q_OBJECT
public:
  MLine(QWidget *parent, bool readonly);
  void SetReadOnly(bool);
  bool ReadOnly(void) { return m_bReadOnly; }
public slots:
  void setEnabled(bool);
protected:
  QColor baseRO, baseRW;
  bool m_bReadOnly;
};

class ITButton : public QPushButton {
  Q_OBJECT
public:
  ITButton(QWidget *parent=0, const char *name=0);
  ~ITButton(void);
public slots:
  void setIT(const QPixmap &, const QString &);
public:
  QPixmap bpix;
  QString btext;
signals:
protected:
  virtual void drawButtonLabel(QPainter *);
protected slots:
private:
private slots:
};
#endif /* COREWIDGET_H */
